<?php
namespace Brix\CelumConnect\Controller;

    /***************************************************************
     *
     *  Copyright notice
     *
     *  (c) 2016
     *
     *  All rights reserved
     *
     *  This script is part of the TYPO3 project. The TYPO3 project is
     *  free software; you can redistribute it and/or modify
     *  it under the terms of the GNU General Public License as published by
     *  the Free Software Foundation; either version 3 of the License, or
     *  (at your option) any later version.
     *
     *  The GNU General Public License can be found at
     *  http://www.gnu.org/copyleft/gpl.html.
     *
     *  This script is distributed in the hope that it will be useful,
     *  but WITHOUT ANY WARRANTY; without even the implied warranty of
     *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     *  GNU General Public License for more details.
     *
     *  This copyright notice MUST APPEAR in all copies of the script!
     ***************************************************************/

/**
 * AssetController
 */


class AssetController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
    /**
     * assetRepository
     *
     * @var \Brix\CelumConnect\Domain\Repository\AssetRepository
     * @inject
     */
    protected $assetRepository = NULL;

    /**
     * action list
     *
     * @return void
     */
    public function listAction() {
        $config = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['celum_connect']);
        if(!isset($GLOBALS['_REQUEST']['id'])){
            $this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_celumconnect.select_target', 'celum_connect'), '',\TYPO3\CMS\Core\Messaging\FlashMessage::INFO,true);
        }else{
	        $extension_path = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath('celum_connect');
	        $file = $extension_path . 'Resources/Public/assetPicker/pickers/assetPicker_2.'.$config['asset_picker_version'].'/config.js';
            $this->createPickerConfig($config, $file);
	        $assets = $this->assetRepository->findAll();
            $this->view->assign('assets', $assets);
        }
        $this->view->assign('asset_picker_version', $config['asset_picker_version']);
    }

	/**
	 * @param $config
	 *
	 * @param $file
	 *
	 * @return int
	 */
    function createPickerConfig($config, $file) {
        $script = 'Custom.AssetPickerConfig = {
            endPoint: \'' . $this->decrypt($config['license']) . '/cora\',
            apiKey: \'' . $config['api_key'] . '\',
            locale: \'' . \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_celumconnect.languagekey', 'celum_connect') . '\',
            searchScope: {
              rootNodes: [' . $config['root_nodes'] . ']
            },
            requiredAssetData: [\'fileInformation\',\'versionInformation\',\'binaries\'],
            downloadFormats: {
              defaults: {
                unknown: ' . ($config['unknown']?$config['unknown']:0) . ',
                image: ' . ($config['image']?$config['image']:0) . ',
                document:' . ($config['document']?$config['document']:0) . ',
                video: ' . ($config['video']?$config['video']:0) . ',
                audio: ' . ($config['audio']?$config['audio']:0) . ',
                text: ' . ($config['text']?$config['text']:0) . '
                },
                supported: [' . $config['supported'] . '],
                additionalDownloadFormats: [' . $config['supported'] . ']
            },
            nrOfAllowedDownloadFormats: 5,
            forceDownloadSelection: true,
            keepSelectionOnExport: true
        };';
	    $fp = fopen($file, "w+");
	    if ( $fp==false) {
		    $this->addFlashMessage( 'Couldn\'t open config file '.$file, '',\TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR, false);
	        $response = false;
	    }else{
		    $response = fwrite($fp, $script);
		    if(!$response){
			    $this->addFlashMessage( 'Couldn\'t write config file '.$file, '',\TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR, false);
		    }
		    fclose($fp);
	    }
	    return $response;
    }

    function decode_base64($sData){
        $sBase64 = strtr($sData, '-_', '+/');
        return base64_decode($sBase64.'==');
    }

    function decrypt($sData){
        $secretKey = "ZbMchtd9DivzjPDi5QIio1iVERFnNZiSE33QKY3Gw9rYfCNLFiKloJQt3zi4";
        $sResult = '';
        $sData   = $this->decode_base64($sData);
        for($i=0;$i<strlen($sData);$i++){
            $sChar    = substr($sData, $i, 1);
            $sKeyChar = substr($secretKey, ($i % strlen($secretKey)) - 1, 1);
            $sChar    = chr(ord($sChar) - ord($sKeyChar));
            $sResult .= $sChar;
        }
        return $sResult;
    }


    /**
     * action create
     *
     * @param \Brix\CelumConnect\Domain\Model\Asset $newAsset
     * @return string
     */
    public function createAction(\Brix\CelumConnect\Domain\Model\Asset $newAsset) {
        if (isset($GLOBALS['_REQUEST']['id'])) {
            $config = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['celum_connect']);
            if ($config['asset_name'] == 1) {
                $filename = $newAsset->getTitle();
            } else {
            	$filename = $newAsset->getFilename();
            }
            $upload_path = PATH_site . 'uploads/celum/';
            if (!file_exists($upload_path)) {
                mkdir($upload_path, 511, true);
            }
            $url = $newAsset->getUrl();
            if (strpos($url, ':443/cora') !== false) {
                $url = str_replace("http://","https://",$url);
            }

            // download the file
            $fp = fopen($url, 'r');
	        $metadata = stream_get_meta_data($fp);
	        foreach($metadata['wrapper_data'] as $header) {
		        if (strstr($header, "Content-Disposition")) {
			        $preg_matches = array(); // try to extract the download file name from the content header
			        preg_match('/Content-Disposition:.*?filename="?([^";]+)/', $header, $preg_matches);
			        if (count($preg_matches) > 1 && $preg_matches[1]) { // use the entire name, or at least set the correct file extension
			        	if ($config['asset_name'] == 2) $filename = $preg_matches[1];
				        else $filename = pathinfo($filename, PATHINFO_FILENAME) .'.'. pathinfo($preg_matches[1], PATHINFO_EXTENSION);
			        }
		        }
	        }
            file_put_contents($upload_path . $filename, $fp);

            //DirectDownload for usage
	        if ($config['usage']) {
		        $directDownloadURL = $this->decrypt($config['license'])."/direct/download?id=".$newAsset->getId()."&format=".$newAsset->getDownloadFormat()."&consumer=".$config['usage'];
		        try {
			        @fopen($directDownloadURL, 'r');
		        }
		        catch (Exception $e) {
		        }
	        }
            $storageRepository = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Resource\\StorageRepository');
            $fileStorageId = explode(':', $GLOBALS['_REQUEST']['id']);

            $storage = $storageRepository->findByUid($fileStorageId[0]);
            $folder = $storage->getFolder($fileStorageId[1]);
            $file = $storage->addFile($upload_path . $filename, $folder, $filename, \TYPO3\CMS\Core\Resource\DuplicationBehavior::REPLACE, true);
            $metaData = array(
                'title' => $filename,
                'description' => $newAsset->getDescription()
            );
            $metaDataRepository = \TYPO3\CMS\Core\Resource\Index\MetaDataRepository::getInstance();
            $metaDataRepository->update($file->getUid(), $metaData);
            $file->_updateMetaDataProperties($metaData);
            $newAsset->setFilename($GLOBALS['_REQUEST']['id'] . $filename);
            $this->assetRepository->add($newAsset);
            $message =  \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_celumconnect.saved', 'celum_connect');
            $response = array('status' => 'success', 'message' => 'Asset "' . $filename . $message );
        } else {
            $message = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_celumconnect.no_target', 'celum_connect');
            $response = array('status' => 'error', 'message' => $message);

        }
        return json_encode($response);
    }

    /**
     * action new
     *
     * @return void
     */
    public function newAction()
    {

    }

}